/* License file snipped from here. Please see the "LICENSE" file, or the top of
 * the "swamp.h" header. Both contain the current license. */


#ifndef HEADER_CONFIGWRAP_H
#define HEADER_CONFIGWRAP_H

/* Include the autoconf-generated feature-test header */
#include "config.h"

/* Run some basic can-work/wont-work tests first */
#if !defined(HAVE_OPENSSL_SSL_H) && !defined(WITH_SSL)
#error "Must ensure openssl headers and libraries are present"
#endif
/* "--with-ssl" results in different symbols and config than autoprobing,
 * correct things. */
#if !defined(SWAMP_WITH_NO_ENGINE_SUPPORT) && \
		!defined(HAVE_OPENSSL_ENGINE_H) && \
		defined(HAVE_OPENSSL_SSL_H)
#define SWAMP_WITH_NO_ENGINE_SUPPORT
#endif
/* We don't implement replacements for these :-) */
#if !defined(HAVE_SELECT) || !defined(HAVE_SOCKET)
#error "Both 'select()' and 'socket()' must be supported"
#endif
/* When we use "recv" and "send", we need to define the 4th parameter */
#ifdef MSG_NOSIGNAL
#define RECV_FLAGS	MSG_NOSIGNAL
#ifdef MSG_DONTWAIT
#define SEND_FLAGS	(MSG_NOSIGNAL | MSG_DONTWAIT)
#else
#define SEND_FLAGS	MSG_NOSIGNAL
#endif
#else
#define RECV_FLAGS	0
#define SEND_FLAGS	0
#endif
/* Make sure we have a "max" macro */
#ifndef max
	#define max(a,b) (((a) < (b)) ? (b) : (a))
#endif

/* Handle replacement functions */
#ifndef HAVE_STRDUP
char *strdup(const char *s);
#endif
/* "strtol" should only be called from inside utils.c, everything else uses util
 * wrapper functions. So 'strtol' is replaced statically in utils.c if it is
 * missing. */
/* long int strtol(const char *nptr, char **endptr, int base); */
/* We don't replace gettimeofday() as only timing.h functions should be used for
 * timing. However, we need a time *structure*, so handle it here. */
#ifdef HAVE_GETTIMEOFDAY
typedef struct timeval timing_t;
#else
typedef time_t timing_t;
#endif

#ifdef HAVE_STDIO_H
#include <stdio.h>
#endif
#ifdef HAVE_ASSERT_H
#include <assert.h>
#endif
#ifdef HAVE_STRING_H
#include <string.h>
#endif
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif
#ifdef HAVE_ERRNO_H
#include <errno.h>
#endif
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#ifdef HAVE_SYS_SOCKET_H
#include <sys/socket.h>
#endif
#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#endif
#ifdef HAVE_NETINET_IN_H
#include <netinet/in.h>
#endif
#ifdef HAVE_NETDB_H
#include <netdb.h>
#endif
/* OpenSSL */
#include <openssl/ssl.h>
#include <openssl/rand.h>
#include <openssl/err.h>
#ifndef SWAMP_WITH_NO_ENGINE_SUPPORT
#   include <openssl/engine.h>
#endif

#endif /* !HEADER_CONFIGWRAP_H */
