/* License file snipped from here. Please see the "LICENSE" file, or the top of
 * the "swamp.h" header. Both contain the current license. */


#ifndef HEADER_FIFO_H
#define HEADER_FIFO_H

#include "configwrap.h"

/*
 * A first-in-first-out buffer structure. Note, the name is to avoid conflicts
 * with what could be a commonly used type/function/macro name.
 *
 */

typedef struct st_swamp_fifo swamp_fifo;

/* Create/destroy a "fifo" */
swamp_fifo *swamp_fifo_new(unsigned int size);
void swamp_fifo_free(swamp_fifo *f);

/* Boolean tests */
int swamp_fifo_isempty(const swamp_fifo *f);
int swamp_fifo_isfull(const swamp_fifo *f);
int swamp_fifo_notempty(const swamp_fifo *f);
int swamp_fifo_notfull(const swamp_fifo *f);

/* Other query functions */
unsigned int swamp_fifo_used(const swamp_fifo *f);
unsigned int swamp_fifo_unused(const swamp_fifo *f);

/* Interaction between "fifo"s and OpenSSL's "BIO"s. Return value is the amount
 * of data moved in each case. */
unsigned int swamp_fifo_from_BIO(swamp_fifo *f, BIO *bio);
unsigned int swamp_fifo_to_BIO(swamp_fifo *f, BIO *bio);

/* Interaction between "fifo"s and file-descriptors. */
ssize_t swamp_fifo_from_fd(swamp_fifo *f, int fd);
ssize_t swamp_fifo_to_fd(swamp_fifo *f, int fd);

#endif /* !defined(HEADER_FIFO_H) */
