/* License file snipped from here. Please see the "LICENSE" file, or the top of
 * the "swamp.h" header. Both contain the current license. */


#ifndef HEADER_NET_H
#define HEADER_NET_H

/* For the "ipv4_parse_error_t" type */
#include "utils.h"
/* For "fifo"s */
#include "fifo.h"

/*
 * Network abstractions.
 *
 * The use of the "swamp_" prefix is to avoid possible name conflicts.
 *
 */

typedef struct st_swamp_address swamp_address;
typedef struct st_swamp_connection swamp_connection;
typedef struct st_swamp_selector swamp_selector;

/***************************/
/* swamp_address functions */
/***************************/

/* Create/destroy an "address" */
swamp_address *swamp_address_new(const char *address_string,
				ipv4_parse_error_t *error_val);
void swamp_address_free(swamp_address *sa);

/* Query functions */
const unsigned char *swamp_address_get_ip(const swamp_address *item);
in_port_t swamp_address_get_port(const swamp_address *item);
const char *swamp_address_get_string(const swamp_address *item);

/******************************/
/* swamp_connection functions */
/******************************/

/* Create/destroy a "connection" */
swamp_connection *swamp_connection_new(const swamp_address *address,
				unsigned int buffer_size);
void swamp_connection_free(swamp_connection *c);
/* Allow two OpenSSL BIOs to read from, and write to, the connection */
void swamp_connection_tofrom_BIO(swamp_connection *c,
				BIO *bio_fromconnection,
				BIO *bio_toconnection);
/* Get (const) access to the incoming and outgoing "fifo"s */
const swamp_fifo *swamp_connection_get_read(swamp_connection *c);
const swamp_fifo *swamp_connection_get_send(swamp_connection *c);

/****************************/
/* swamp_selector functions */
/****************************/

/* Create/destroy a "selector" */
swamp_selector *swamp_selector_new(void);
void swamp_selector_free(swamp_selector *ss);

/* Perform a select operation */
int swamp_selector_select(swamp_selector *ss, int use_timeout,
				unsigned long timeout_msecs);

/* Add a connection to a selector (zero return for connection failure) */
int swamp_selector_add_connection(swamp_selector *ss,
				const swamp_connection *c);

/* Perform connection I/O after a select (zero return for connection failure */
int swamp_selector_connection_io(swamp_selector *ss,
				swamp_connection *c);

#endif /* !defined(HEADER_NET_H) */
