#!/bin/sh

# Set the package name by slurping out the line of configure.in that
# specifies the version number, eg;
#
# AM_INIT_AUTOMAKE(swamp, <version>)
VERSION=`cat configure.in | egrep "^AM_INIT_AUTOMAKE" | sed -e \
	"s/^.*, //" | sed -e "s/)$//"`
# We form a tarball in pieces then gzip it distinctly afterwards
PACKAGE="swamp-$VERSION.tar"
# A temporary directory we can create/delete/etc
TDIR=.temppackagedir
# A temporary file we put our package list in
PLIST=.packagelist

if [ -f "$PACKAGE" -o -f "$PACKAGE.gz" ]; then
	echo "Either $PACKAGE or $PACKAGE.gz already exists, quitting"
	exit 1
fi

# Ensure "cvs_finish.sh" has cleaned things up to its satisfaction
./cvs_finish.sh || exit 1;

# Now, generate the necessary derivative files so we can build on systems
# without autoconf/automake/etc.
./cvs_start.sh || exit 1;

# Prepare a list of files to package
#
# Steps;
# (1) Remove CVS entries
# (2) Remove dotfiles
# (3) Remove leading "./"
# (4) Remove "TODO"
# (5) Remove 'README' - we'll use that as the fixed first entry
# (6) Remove "sludge" files

find . -type f | \
	grep -v "CVS/" | \
	egrep -v "\/\." | \
	sed -e "s/^\.\///" | \
	grep -v "TODO" | \
	grep -v "README" | \
	egrep -v "core$" | \
	egrep -v "\.gz$" | \
	egrep -v "\.tar$" > $PLIST

tar --create --file="$PACKAGE" README || exit 1;

(cat $PLIST | xargs tar --append --file="$PACKAGE") || exit 1;

# Now create a temporary store, extract the tarball into it, then recreate
# a tarball with the appropriate directory structure
(rm -rf $TDIR && mkdir $TDIR && cd $TDIR && mkdir "swamp-$VERSION" && \
	cd "swamp-$VERSION" && tar --extract --file="../../$PACKAGE" && \
	cd .. && tar --gzip --create --file="../$PACKAGE.gz" swamp* && \
	cd .. && rm -rf $TDIR && rm -f "$PACKAGE") || exit 1;

rm -f $PLIST

# Success!
echo "$PACKAGE.gz created"
