/* License file snipped from here. Please see the "LICENSE" file, or the top of
 * the "swamp.h" header. Both contain the current license. */


/* For "dist_pattern" */
#include "dist_pattern.h"

/* Recognise our functions */
#include "serv_iterator.h"

/* This type represents a "cursor" to iterate over a dist_pattern */
struct st_server_iterator {
	dist_pattern *p;
	unsigned int idx;
};

/*****************************/
/* server_iterator functions */
/*****************************/

server_iterator *server_iterator_new(dist_pattern *p)
{
	server_iterator *c = malloc(sizeof(server_iterator));
	if(!c)
		return NULL;

	/* Grab a reference to the dist_pattern */
	dist_pattern_up(p);

	c->p = p;
	c->idx = dist_pattern_get_start_idx(p);

	return c;
}

void server_iterator_free(server_iterator *c)
{
	/* Drop our reference to the pattern */
	dist_pattern_free(c->p);
	free(c);
}

const swamp_address *server_iterator_next(server_iterator *c)
{
	/* Get the index of the item in the pattern we want */
	unsigned int idx = c->idx;

	/* Adjust the index so next time it points to the next item in the
	 * pattern */
	if(++(c->idx) >= dist_pattern_period(c->p))
		c->idx = 0;

	/* Return the 'idx'th server in the pattern to the caller. */
	return dist_pattern_get(c->p, idx);
}

