/*
 * Copyright Geoff Thorpe, 2001. All rights reserved.
 * With thanks to Red Hat and Cryptographic Appliances.
 *
 * This tool is free for commercial and non-commercial use as long as
 * the following conditions are aheared to. (NB: the following conditions
 * apply to all code found in this distribution).
 *
 * Copyright remains Geoff Thorpe's, and as such any Copyright notices in
 * the code are not to be removed. If this package is used in a product,
 * Geoff Thorpe should be given attribution as the author of the parts of
 * the software used (whether in source code or compiled form). This can
 * be in the form of a textual message at program startup or in
 * documentation (online or textual) provided with the package.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the copyright
 *    notice, this list of conditions, and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. All advertising materials mentioning details/specifications of this
 *    software, or mentioning other third-party attributions or copyright
 *    notices, must display the following acknowledgement:
 *    "This product includes software written by Geoff Thorpe
 *    (geoff@geoffthorpe.net). With thanks to Red Hat (www.redhat.com)
 *    and Cryptographic Appliances (www.cryptoapps.com)."
 *
 * THIS SOFTWARE IS PROVIDED BY GEOFF THORPE ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The licence and distribution terms for any publically available version or
 * derivative of this code cannot be changed. I.e. this code cannot simply be
 * copied and put under another distribution licence
 * [including the GNU Public Licence.]
 */

#ifndef HEADER_SWAMP_H
#define HEADER_SWAMP_H

/* Learn about "swamp_connection" */
#include "net.h"

/* Learn about "swamp_config" */
#include "swamp_conf.h"

#include "serv_iterator.h"

/*********************************************************/
/* types and defines implemented/used in;    swamp.c     */
/*********************************************************/

/* Predeclare the "parent" type */
typedef struct st_swamp_thread_ctx	swamp_thread_ctx;

/* An individual "swamp_item". Each one of these cycles through the loop of
 * connecting to a server, performing the SSL/TLS handshake, sending a request,
 * reading a response, and closing the connection. */
typedef struct st_swamp_item {
	/* The parent "thread_ctx" that owns us */
	const swamp_thread_ctx *parent;
	/********************************************************/
	/* The SSL/TLS, http(s), request/response "state" stuff */
	/********************************************************/
	SSL *ssl;
	/* Last used session, for resumes. */
	SSL_SESSION *ssl_sess;
	/* The BIOs hooked into the 'dirty' side of 'ssl'. */
	BIO *bio_in;
	BIO *bio_out;
	/* A buffer for the clear-text request we're sending */
	const unsigned char *request;
	unsigned int request_size;
	unsigned int request_sent;
	/* A buffer for the clear-text response we're reading */
	unsigned char *response;
	unsigned int response_size;
	unsigned int response_received;
	unsigned int response_expected;
	/********************************/
	/* The connection to the server */
	/********************************/
	swamp_connection *conn;
	/***************************************/
	/* The iterator across the server list */
	/***************************************/
	server_iterator *server_iterator;
	/*********/
	/* Stats */
	/*********/
	unsigned int total_completed;
	unsigned int total_failed;
	unsigned int handshake_complete;
	unsigned int resumes_hit;
	unsigned int resumes_missed;
} swamp_item;

/* A context for managing multiple "swamp_item"s in a single-thread
 * single-process async-I/O manner. */
struct st_swamp_thread_ctx {
	/* The configuration we work from */
	const swamp_config *config;
	/* SSL_CTX containing our settings and OpenSSL state */
	SSL_CTX *ssl_ctx;
	/* The list of swamp items */
	swamp_item *items;
	unsigned int size;
	/* The selector used for async-I/O */
	swamp_selector *sel;
	/* Collected stats from across all swamp items */
	unsigned int total_completed;
	unsigned int total_failed;
	unsigned int total_max;
	unsigned int resumes_hit;
	unsigned int resumes_missed;
};

#endif /* !defined(HEADER_SWAMP_H) */
