/* License file snipped from here. Please see the "LICENSE" file, or the top of
 * the "swamp.h" header. Both contain the current license. */


#include "text_msg.h"

static unsigned int verify_fail_reported = 0; /* Certificate-verification failures
						 are reported only once. */

void copyright(int nologo)
{
	if(!nologo)
		printf("\nThis is SWAMP (%s), an OpenSSL-based SSL/TLS load-tester\n"
			"Copyright (c) 2001 Geoff Thorpe, http://www.geoffthorpe.net/\n"
			"With thanks to Red Hat (http://www.redhat.com/) and\n"
			"Cryptographic Appliances (http://www.cryptoapps.com/)\n\n",
			VERSION);
}

void main_usage(void)
{
	printf("usage: swamp [options ...]    where the options are\n");
	printf("    -connect <host>:<port>\n");
	printf("                      - (REQUIRED) a host and port to connect to\n");
	printf("    -CAfile <path>    - a PEM file containing trusted CA certificates\n");
	printf("                        (default = a list of common global paths)\n");
	printf("    -cert <path>      - a PEM file containing the client cert and key\n");
	printf("                        (default, no client certificate)\n");
	printf("    -num <n>          - the number of simultaneous connections to use\n");
	printf("                        (default = 5)\n");
	printf("    -count <n>        - the maximum number of requests to count\n");
	printf("                        (default = 0, keep going indefinitely)\n");
	printf("    -time <n>         - the maximum number of seconds to run for\n");
	printf("                        (default = 0, keep going indefinitely)\n");
	printf("    -request <string> - the string to send to the server\n");
	printf("                        (default = \"GET /\\r\\n\")\n");
	printf("    -expect <n>       - the amount of data to expect in a response\n");
	printf("                        (default = 20 bytes)\n");
	printf("    -session <string> - a string of 's' (new session) and 'r' (resume)\n");
	printf("                        (default = \"s\")\n");
	printf("    -update <n>       - the number of seconds between printed updates\n");
	printf("                        (default = 0, no printed updates)\n");
	printf("    -cipher <string>  - a string specifying the cipher suites\n");
	printf("                        (default = 0, assume OpenSSL defaults)\n");
	printf("    -csv <path>       - output per-second summaries to a CSV file\n");
	printf("                        (default, no CSV output)\n");
	printf("    -session_ids      - display all SSL session IDs negotiated\n");
#ifndef SWAMP_WITH_NO_ENGINE_SUPPORT
	printf("    -engine <id>      - Initialise and use the specified engine\n");
#endif
	printf("    -distribute <str> - a pattern of server indexes for distribution\n");
	printf("    -nologo           - supresses the output of the program name and author\n");
	printf("                        (useful when trying to parse output automatically)\n");
	printf("    -h, -? or --help  - display version number and usage information\n");
}

int unknown_switch(const char *str)
{
	fprintf(stderr, 
		"invalid switch \"%s\" (or invalid value). See 'swamp -h' for usage\n",
		str);
	return 0;
}

void verify_result_warning(void)
{
	if(verify_fail_reported)
		return;
	fprintf(stderr,
"Certificate verification failed, probably a self-signed server cert *or*\n"
"the signing CA cert is not trusted by us (hint: use '-CAfile').\n"
"This message will only be printed once\n");
	verify_fail_reported = 1;
}

/* This function gains some code clarity in main() */
int openssl_err(void)
{
	fprintf(stderr, " (OpenSSL errors follow)\n");
	ERR_print_errors_fp(stderr);
	return 1;
}
