/* License file snipped from here. Please see the "LICENSE" file, or the top of
 * the "swamp.h" header. Both contain the current license. */


#include "timing.h"

int int_gettime(timing_t *t)
{
#ifdef HAVE_GETTIMEOFDAY
	return (gettimeofday(t, NULL) ? 0 : 1);
#else
	return (time(t) == (time_t)-1 ? 0 : 1);
#endif
}

void int_cpytime(timing_t *dest, const timing_t *src)
{
	/* This version should handle any version of timing_t */
	memcpy(dest, src, sizeof(timing_t));
}

long int_msecs_since(const timing_t *t)
{
	long res;
	timing_t now;
	if(!int_gettime(&now))
		return -1;
	if((res = int_msecs_between(t, &now)) < 0)
		res = -1;
	return res;
}

long int_msecs_between(const timing_t *then, const timing_t *now)
{
#ifdef HAVE_GETTIMEOFDAY
	long res = now->tv_sec - then->tv_sec;
	res *= 1000000;
	if(now->tv_usec > then->tv_usec)
		res += (now->tv_usec - then->tv_usec);
	else
		res -= (then->tv_usec - now->tv_usec);
	res /= 1000;
	return res;
#else
	return ((*now - *then) * 1000);
#endif
}
