/* License file snipped from here. Please see the "LICENSE" file, or the top of
 * the "swamp.h" header. Both contain the current license. */


#ifndef HEADER_UTILS_H
#define HEADER_UTILS_H

#include "configwrap.h"

/******************/
/* tokeniser code */
/******************/

typedef struct {
	const char *string;
	const char *delimiters;
	const char *position;
	char *token;
} tokeniser_t;

int init_tokeniser(tokeniser_t *t, const char *string, const char *delimiters);
void free_tokeniser(tokeniser_t *t);
char *do_tokenising(tokeniser_t *t);

/************************************/
/* Common library wrapper functions */
/************************************/

/* Wrap strtol so we (a) get error handling automatically, (b) don't have to *
 * specify a base, and (c) could build easily on platforms without strol(). */ 
int int_strtol(const char *str, long *val);

/* Wrap strtoul in the same way, except we also return failure if the number
 * parsed was in fact negative. */
int int_strtoul(const char *str, unsigned long *val);

/* Substring variants. These work the same as the above except it is not assumed
 * the string up to the NULL-terminator must be entirely numeric, parsing can
 * terminate on any non-numeric character, if valid_terms is NULL *or* on the
 * first character from the valid_terms string. If valid_terms is non-NULL and
 * parsing encounters a non-numeric character not in valid_terms, the result is
 * failure. */
int int_substrtol(const char *str, long *val, const char *valid_terms);
int int_substrtoul(const char *str, unsigned long *val, const char *valid_terms);

/**********************************/
/* Parsing and manipulation tools */
/**********************************/

/* Understands the "<hostname-or-ipv4dotted>:<port>" format and places the IP
 * address (byte-array, not uint32) and port into the provided locations if
 * successful. */
typedef enum {
	/* Success */
	ERR_IPV4_PARSE_OKAY = 0,
	/* Parsing couldn't follow the syntax */
	ERR_IPV4_PARSE_INVALID_SYNTAX,
	/* The port number was invalid or out of range */
	ERR_IPV4_PARSE_INVALID_PORT,
	/* The host name was invalid or couldn't be resolved */
	ERR_IPV4_PARSE_INVALID_HOST,
	/* A memory error took place */
	ERR_IPV4_PARSE_MEM_FAILURE,
	/* An error that wrapper functions can use if they want */
	ERR_IPV4_PARSE_ARRAY_FULL
} ipv4_parse_error_t;
ipv4_parse_error_t util_parse_ipv4_address(const char *str_address,
				unsigned char *ip_address,
				in_port_t *ip_port);

/* Take a string input and return a copy with all "un-escaping" performed. Ie.
 * the literal string "hello\nI am fine\n" is converted to a string with two
 * line-feed control-characters. */
char *util_parse_escaped_string(const char *str_toconvert);

#endif /* HEADER_UTILS_H */
